@@ -134,6 +134,7 @@ from __future__ import division, print_function
 __all__ = ["Dream"]
 
 import sys
+import os
 import time
 from ctypes import c_double
 
@@ -147,6 +148,8 @@ from .diffev import de_step
 from .bounds import make_bounds_handler
 from .compiled import dll
 from .util import rng
+from .convergence import ks_converged
+from .outliers import identify_outliers
 
 # Everything should be available in state, but lets be lazy for now
 LAST_TIME = 0
@@ -179,8 +182,11 @@ class Dream(object):
     burn = 0
     draws = 100000
     thinning = 1
-    outlier_test = "IQR"
+    # TODO: change the default outlier test to IQR and control with options
+    outlier_test = "none"
     population = None
+    #: convergence criteria
+    alpha = 0.01
     # DE parameters
     DE_steps = 10
     DE_pairs = 3
@@ -278,8 +284,7 @@ def _run_dream(dream, abort_test=lambda: False):
                               force_keep=True)
             dream.monitor(state, x, logp)
 
-    # Skip r_stat and pCR until we have some data data to analyze
-    state._update(R_stat=-2, CR_weight=dream.CR.weight)
+    state._update(CR_weight=dream.CR.weight)
 
     # Now start drawing samples
     #print "previous draws", previous_draws, "new draws", dream.draws+dream.burn
@@ -291,7 +296,6 @@ def _run_dream(dream, abort_test=lambda: False):
         xtry = np.empty((n_chain, n_var), 'd')
         step_alpha = np.empty(n_chain, 'd')
         CR_used = np.empty(n_chain, 'd')
-    #need_outliers_removed = True
     scale = 1.0
     #serial_time = parallel_time = 0.
     #last_time = time.time()
@@ -301,7 +305,11 @@ def _run_dream(dream, abort_test=lambda: False):
     pop = state._draw_pop()
     assert pop.ctypes.data == np.ascontiguousarray(pop).ctypes.data
 
-    while state.draws < dream.draws + dream.burn:
+    frame = 0
+    next_outlier_test = max(state.Ngen, 2*state.Ngen - 10)
+    next_convergence_test = state.Ngen
+    final_gen = dream.draws + dream.burn
+    while state.draws < final_gen:
 
         # Age the population using differential evolution
         dream.CR.reset()
@@ -412,20 +420,16 @@ def _run_dream(dream, abort_test=lambda: False):
         # End of differential evolution aging
         # ---------------------------------------------------------------------
 
-        # Calculate Gelman and Rubin convergence diagnostic
-        #_, points, _ = state.chains()
-        #r_stat = gelman(points, portion=0.5)
-        r_stat = 0.  # Suppress for now since it is broken, and it costs to unroll
+        # Save update information
+        state._update(CR_weight=dream.CR.weight)
+
+        if abort_test():
+            break
 
         #if state.draws <= 0.1 * dream.draws:
         if state.draws <= dream.burn:
             # Adapt the crossover ratio, but only during burn-in.
             dream.CR.adapt()
-        # See whether there are any outlier chains, and remove
-        # them to current best value of X
-        #if need_outliers_removed and state.draws > 0.5*dream.burn:
-        #    state.remove_outliers(x, logp, test=dream.outlier_test)
-        #    need_outliers_removed = False
 
         if False:
             # Suppress scale update until we have a chance to verify that it
@@ -437,14 +441,43 @@ def _run_dream(dream, abort_test=lambda: False):
             elif ravg < 0.2:
                 scale /= 1.01
 
+        if state.generation >= next_convergence_test:
+            converged = ks_converged(state, alpha=dream.alpha)
+        else:
+            converged = False
 
+        # See whether there are any outlier chains, and remove them
+        # Only do this once per frame, and only if there is some time
+        # left to adapt the distribution.  Also do it if we believe
+        # that we have converged.
+        if ((converged or state.generation >= next_outlier_test)
+                and state.generation + state.Ngen < final_gen):
+            outliers = state.remove_outliers(x, logp, dream.outlier_test)
+            next_outlier_test = state.generation + 2*state.Ngen
+            if len(outliers):
+                # TODO: use monitors to report arbitrary information
+                print("step %d trimmed %d outliers from %d chains"
+                      % (state.generation, len(outliers), len(logp)))
+                next_convergence_test = state.generation + state.Ngen
+                converged = False
 
-        # Save update information
-        state._update(R_stat=r_stat, CR_weight=dream.CR.weight)
-
-        if abort_test():
+        if converged:
+            #_show_logp_frame(dream, state, frame+1)
             break
 
+        # Draw the next frame (for debugging...)
+        next_frame = state.generation // state.Ngen
+        #if frame != next_frame: _show_logp_frame(dream, state, next_frame)
+        frame = next_frame
+
+def _show_logp_frame(dream, state, frame):
+    from pylab import clf, savefig
+    from . import views
+    clf()
+    views.plot_logp(state)
+    savefig('logp%03d.png'%frame)
+    clf()
+
 
 def allocate_state(dream):
     """
