@@ -11,6 +11,7 @@ Users can also perform calculations with parameters, tying together different
 parts of the model, or different models.
 """
 #__all__ = [ 'Parameter']
+import operator
 import sys
 from six.moves import reduce
 import warnings
@@ -192,15 +193,15 @@ class BaseParameter(object):
 
     # Parameter algebra: express relationships between parameters
     def __gt__(self, other):
-        return ConstraintGT(self, other)
+        return Constraint(self, other, "GT", ">")
 
     def __ge__(self, other):
-        return ConstraintGE(self, other)
+        return Constraint(self, other, "GE", ">=")
     def __le__(self, other):
-        return ConstraintLE(self, other)
+        return Constraint(self, other, "LE", "<=")
 
     def __lt__(self, other):
-        return ConstraintLT(self, other)
+        return Constraint(self, other, "LT", "<")
 
     # def __eq__(self, other):
     #     return ConstraintEQ(self, other)
@@ -209,34 +210,34 @@ class BaseParameter(object):
     #     return ConstraintNE(self, other)
 
     def __add__(self, other):
-        return OperatorAdd(self, other)
+        return Operator(self, other, "add", "+")
 
     def __sub__(self, other):
-        return OperatorSub(self, other)
+        return Operator(self, other, "sub", "-")
 
     def __mul__(self, other):
-        return OperatorMul(self, other)
+        return Operator(self, other, "mul", "*")
 
     def __div__(self, other):
-        return OperatorDiv(self, other)
+        return Operator(self, other, "truediv", "/")
 
     def __pow__(self, other):
-        return OperatorPow(self, other)
+        return Operator(self, other, "pow", "**")
 
     def __radd__(self, other):
-        return OperatorAdd(other, self)
+        return Operator(other, self, "add", "+")
 
     def __rsub__(self, other):
-        return OperatorSub(other, self)
+        return Operator(other, self, "sub", "-")
 
     def __rmul__(self, other):
-        return OperatorMul(other, self)
+        return Operator(other, self, "mul", "*")
 
     def __rdiv__(self, other):
-        return OperatorDiv(other, self)
+        return Operator(other, self, "truediv", "/")
 
     def __rpow__(self, other):
-        return OperatorPow(other, self)
+        return Operator(other, self, "pow", "**")
 
     def __abs__(self):
         return _abs(self)
@@ -680,63 +681,43 @@ class FreeVariables(object):
 #
 
 # ==== Comparison operators ===
+COMPARISONS = [
+    ('GT', '>'),
+    ('GE', '>='),
+    ('LE', '<='),
+    ('LT', '<'),
+    ('EQ', '=='),
+    ('NE', '!=')
+]
+
 class Constraint(object):
-    """
-    Abstract base class for constraints.
-    """
-
+    def __init__(self, a, b, op_name, op_str=""):
+        import operator
+        self.a, self.b = a, b
+        self.op_name = op_name
+        self.op = getattr(operator, op_name.lower())
+        self.op_str = op_str
     def __bool__(self):
-        """
-        Returns True if the condition is satisfied
-        """
-        raise NotImplementedError
-    __nonzero__ = __bool__
-
-    def __str__(self):
-        """
-        Text description of the constraint
-        """
-        raise NotImplementedError
-
-
-def _gen_constraint(name, op):
-    """
-    Generate a comparison function from a comparison operator.
-    """
-    return '''\
-class Constraint%(name)s(Constraint):
-    """
-    Constraint operator %(op)s
-    """
-    def __init__(self, a, b):
-        self.a, self.b = a,b
-    def __bool__(self):
-        return float(self.a) %(op)s float(self.b)
+        return self.op(float(self.a), float(self.b))
     __nonzero__ = __bool__
     def __str__(self):
-        return "(%%s %(op)s %%s)"%%(self.a,self.b)
-''' % dict(name=name, op=op)
-
-exec(_gen_constraint('GT', '>'))
-exec(_gen_constraint('GE', '>='))
-exec(_gen_constraint('LE', '<='))
-exec(_gen_constraint('LT', '<'))
-exec(_gen_constraint('EQ', '=='))
-exec(_gen_constraint('NE', '!='))
-
+        return "(%s %s %s)" %(self.a, self.op_str, self.b)
 
 # ==== Arithmetic operators ===
-def _gen_binop(name, op):
+ALLOWED_OPERATORS = ["add","sub","mul","truediv","floordiv","pow"]
+
+class Operator(BaseParameter):
     """
-    Generate a comparison function from a comparison operator.
+    Parameter operator
     """
-    return '''\
-class Operator%(name)s(BaseParameter):
-    """
-    Parameter operator %(op)s
-    """
-    def __init__(self, a, b):
+    def __init__(self, a, b, op_name, op_str):
+        import operator
+        if not op_name.lower() in ALLOWED_OPERATORS:
+            raise ValueError("Operator name %s is not in allowed operators: %s" % (op_name, str(ALLOWED_OPERATORS)))
         self.a, self.b = a,b
+        self.op_name = op_name
+        self.op = getattr(operator, op_name.lower())
+        self.op_str = op_str
         pars = []
         if isinstance(a,BaseParameter): pars += a.parameters()
         if isinstance(b,BaseParameter): pars += b.parameters()
@@ -746,26 +727,20 @@ class Operator%(name)s(BaseParameter):
         return self._parameters
     def to_dict(self):
         return dict(
-            type="Operator%(name)s",
+            type="Operator",
+            op_name=self.op_name,
+            op_str=self.op_str,
             left=to_dict(self.a),
             right=to_dict(self.b),
         )
     @property
     def value(self):
-        return float(self.a) %(op)s float(self.b)
+        return self.op(float(self.a), float(self.b))
     @property
     def dvalue(self):
         return float(self.a)
     def __str__(self):
-        return "(%%s %(op)s %%s)"%%(self.a,self.b)
-''' % dict(name=name, op=op)
-
-exec(_gen_binop('Add', '+'))
-exec(_gen_binop('Sub', '-'))
-exec(_gen_binop('Mul', '*'))
-exec(_gen_binop('Div', '/'))
-exec(_gen_binop('Pow', '**'))
-
+        return "(%s %s %s)" % (self.a,self.op_str, self.b)
 
 def substitute(a):
     """
@@ -1154,3 +1129,11 @@ class Alias(object):
             'obj': to_dict(self.obj),
             'attr': self.attr,
         }
+
+def test_operator():
+    a = Parameter(1, name='a')
+    b = Parameter(2, name='b')
+    a_b = a + b
+    a.value = 3
+    assert a_b.value == 5.
+    assert a_b.name == '(a + b)'
\ No newline at end of file
